@@ -40,6 +40,7 @@ bool_t      xdr_chpass3_arg(XDR *xdrs, chpass3_arg *objp);
 bool_t      xdr_setv4key_arg(XDR *xdrs, setv4key_arg *objp);
 bool_t      xdr_setkey_arg(XDR *xdrs, setkey_arg *objp);
 bool_t      xdr_setkey3_arg(XDR *xdrs, setkey3_arg *objp);
+bool_t      xdr_setkey4_arg(XDR *xdrs, setkey4_arg *objp);
 bool_t	    xdr_chrand_arg(XDR *xdrs, chrand_arg *objp);
 bool_t      xdr_chrand3_arg(XDR *xdrs, chrand3_arg *objp);
 bool_t	    xdr_chrand_ret(XDR *xdrs, chrand_ret *objp);
@@ -68,3 +69,6 @@ bool_t	    xdr_krb5_keyblock(XDR *xdrs, krb5_keyblock *objp);
 bool_t      xdr_krb5_key_data(XDR *xdrs, krb5_key_data *objp);
 bool_t      xdr_krb5_string_attr(XDR *xdrs, krb5_string_attr *objp);
 bool_t      xdr_osa_pw_hist_ent(XDR *xdrs, osa_pw_hist_ent *objp);
+bool_t      xdr_kadm5_key_data(XDR *xdrs, kadm5_key_data *objp);
+bool_t      xdr_getpkeys_arg(XDR *xdrs, getpkeys_arg *objp);
+bool_t      xdr_getpkeys_ret(XDR *xdrs, getpkeys_ret *objp);
