@@ -9,6 +9,7 @@
  *
  */
 #include "k5-int.h"
+#include "int-proto.h"
 #include "cleanup.h"
 #include "auth_con.h"
 
@@ -44,16 +45,9 @@ encrypt_credencpart(krb5_context context, krb5_cred_enc_part *pcredpart,
     }
 
     /* call the encryption routine */
-    retval = krb5_encrypt_keyhelper(context, pkey,
-                                    KRB5_KEYUSAGE_KRB_CRED_ENCPART,
-                                    scratch, pencdata);
-
-    if (retval) {
-        memset(pencdata->ciphertext.data, 0, pencdata->ciphertext.length);
-        free(pencdata->ciphertext.data);
-        pencdata->ciphertext.length = 0;
-        pencdata->ciphertext.data = 0;
-    }
+    retval = k5_encrypt_keyhelper(context, pkey,
+                                  KRB5_KEYUSAGE_KRB_CRED_ENCPART, scratch,
+                                  pencdata);
 
     memset(scratch->data, 0, scratch->length);
     krb5_free_data(context, scratch);
@@ -97,7 +91,7 @@ krb5_mk_ncred_basic(krb5_context context,
      * structure and copy the ticket into the ticket list.
      */
     for (i = 0; i < nppcreds; i++) {
-        credenc.ticket_info[i] = malloc(sizeof(krb5_cred_info));
+        credenc.ticket_info[i] = calloc(1, sizeof(krb5_cred_info));
         if (credenc.ticket_info[i] == NULL) {
             retval = ENOMEM;
             goto cleanup;
