@@ -100,7 +100,7 @@ static const krb5_ser_entry krb5_profile_ser_entry = {
 static inline unsigned int
 etypes_len(krb5_enctype *list)
 {
-    return (list == NULL) ? 0 : krb5int_count_etypes(list);
+    return (list == NULL) ? 0 : k5_count_etypes(list);
 }
 
 /*
@@ -169,7 +169,7 @@ krb5_context_size(krb5_context kcontext, krb5_pointer arg, size_t *sizep)
         *sizep += required;
     return(kret);
 }
-
+
 /*
  * krb5_context_externalize()   - Externalize the krb5_context.
  */
@@ -189,7 +189,8 @@ krb5_context_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet **b
     context = (krb5_context) arg;
     if (!context)
         return (EINVAL);
-    KRB5_VERIFY_MAGIC(context, KV5M_CONTEXT);
+    if (context->magic != KV5M_CONTEXT)
+        return (KV5M_CONTEXT);
 
     if ((kret = krb5_context_size(kcontext, arg, &required)))
         return (kret);
@@ -334,7 +335,7 @@ krb5_context_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet **b
 
     return (0);
 }
-
+
 /*
  * krb5_context_internalize()   - Internalize the krb5_context.
  */
@@ -512,7 +513,7 @@ cleanup:
         krb5_free_context(context);
     return(kret);
 }
-
+
 /*
  * krb5_oscontext_size()        - Determine the size required to externalize
  *                                the krb5_os_context.
@@ -528,7 +529,7 @@ krb5_oscontext_size(krb5_context kcontext, krb5_pointer arg, size_t *sizep)
     *sizep += (5*sizeof(krb5_int32));
     return(0);
 }
-
+
 /*
  * krb5_oscontext_externalize() - Externalize the krb5_os_context.
  */
@@ -565,7 +566,7 @@ krb5_oscontext_externalize(krb5_context kcontext, krb5_pointer arg, krb5_octet *
     }
     return(kret);
 }
-
+
 /*
  * krb5_oscontext_internalize() - Internalize the krb5_os_context.
  */
@@ -618,7 +619,7 @@ krb5_oscontext_internalize(krb5_context kcontext, krb5_pointer *argp, krb5_octet
     }
     return(kret);
 }
-
+
 /*
  * Register the context serializers.
  */
