@@ -31,8 +31,6 @@
  * Create / Delete / Modify / View / List policy objects.
  */
 
-#include <stdio.h>
-#include <time.h>
 #include <k5-int.h>
 #include <kadm5/admin.h>
 #include "kdb5_ldap_util.h"
@@ -799,7 +797,6 @@ kdb5_ldap_list_policies(int argc, char *argv[])
     char *me = progname;
     krb5_error_code retval = 0;
     krb5_boolean print_usage = FALSE;
-    char *basedn = NULL;
     char **list = NULL;
     char **plist = NULL;
 
@@ -811,7 +808,7 @@ kdb5_ldap_list_policies(int argc, char *argv[])
     if ((retval = init_ldap_realm (argc, argv)))
         goto cleanup;
 
-    retval = krb5_ldap_list_policy(util_context, basedn, &list);
+    retval = krb5_ldap_list_policy(util_context, NULL, &list);
     if ((retval != 0) || (list == NULL))
         goto cleanup;
 
@@ -830,9 +827,6 @@ cleanup:
         free (list);
     }
 
-    if (basedn)
-        free (basedn);
-
     if (print_usage) {
         db_usage(LIST_POLICY);
     }
