@@ -40,9 +40,12 @@ static krb5_preauthtype otp_pa_type_list[] =
   { KRB5_PADATA_OTP_REQUEST, 0 };
 
 struct request_state {
+    krb5_context context;
     krb5_kdcpreauth_verify_respond_fn respond;
     void *arg;
     krb5_enc_tkt_part *enc_tkt_reply;
+    krb5_kdcpreauth_callbacks preauth_cb;
+    krb5_kdcpreauth_rock rock;
 };
 
 static krb5_error_code
@@ -151,9 +154,11 @@ nonce_generate(krb5_context ctx, unsigned int length, krb5_data *nonce_out)
 }
 
 static void
-on_response(void *data, krb5_error_code retval, otp_response response)
+on_response(void *data, krb5_error_code retval, otp_response response,
+            char *const *indicators)
 {
     struct request_state rs = *(struct request_state *)data;
+    char *const *ind;
 
     free(data);
 
@@ -163,6 +168,9 @@ on_response(void *data, krb5_error_code retval, otp_response response)
     if (retval == 0)
         rs.enc_tkt_reply->flags |= TKT_FLG_PRE_AUTH;
 
+    for (ind = indicators; ind != NULL && *ind != NULL && retval == 0; ind++)
+        retval = rs.preauth_cb->add_auth_indicator(rs.context, rs.rock, *ind);
+
     rs.respond(rs.arg, retval, NULL, NULL, NULL);
 }
 
@@ -305,9 +313,12 @@ otp_verify(krb5_context context, krb5_data *req_pkt, krb5_kdc_req *request,
     rs = k5alloc(sizeof(struct request_state), &retval);
     if (rs == NULL)
         goto error;
+    rs->context = context;
     rs->arg = arg;
     rs->respond = respond;
     rs->enc_tkt_reply = enc_tkt_reply;
+    rs->preauth_cb = cb;
+    rs->rock = rock;
 
     /* Get the principal's OTP configuration string. */
     retval = cb->get_string(context, rock, "otp", &config);
@@ -320,7 +331,8 @@ otp_verify(krb5_context context, krb5_data *req_pkt, krb5_kdc_req *request,
 
     /* Send the request. */
     otp_state_verify((otp_state *)moddata, cb->event_context(context, rock),
-                     request->client, config, req, on_response, rs);
+                     cb->client_name(context, rock), config, req, on_response,
+                     rs);
     cb->free_string(context, rock, config);
 
     k5_free_pa_otp_req(context, req);
