@@ -1,4 +1,3 @@
-#!/usr/bin/python
 from k5test import *
 
 offline = (len(args) > 0 and args[0] != "no")
@@ -40,21 +39,23 @@ def testu(host, princhost, princrealm):
 
 # With the unknown principal type, we do not canonicalize or downcase,
 # but we do remove a trailing period and look up the realm.
+mark('unknown type')
 testu('ptr-mismatch.kerberos.org', 'ptr-mismatch.kerberos.org', 'R1')
 testu('Example.COM', 'Example.COM', 'R2')
 testu('abcde', 'abcde', '')
 
-# A ':port' or ':instance' trailer should be ignored for hostname
-# adjustment and realm lookup.  If there is more than one colon in the
-# name, we assume it's an IPv6 address and don't treat it as having a
-# trailer.
-testu('example.com.:123', 'example.com:123', 'R2')
+# A ':port' or ':instance' trailer should be ignored for realm lookup.
+# If there is more than one colon in the name, we assume it's an IPv6
+# address and don't treat it as having a trailer.
+mark('port trailer')
+testu('example.com.:123', 'example.com.:123', 'R2')
 testu('Example.COM:xyZ', 'Example.COM:xyZ', 'R2')
 testu('example.com.::123', 'example.com.::123', '')
 
 # With dns_canonicalize_hostname=false, we downcase and remove
 # trailing dots but do not canonicalize the hostname.  Trailers do not
 # get downcased.
+mark('dns_canonicalize_host=false')
 testnc('ptr-mismatch.kerberos.org', 'ptr-mismatch.kerberos.org', 'R1')
 testnc('Example.COM', 'example.com', 'R2')
 testnc('abcde', 'abcde', '')
@@ -69,7 +70,6 @@ if offline:
 # and reverse resolving to these names.
 oname = 'ptr-mismatch.kerberos.org'
 fname = 'www.kerberos.org'
-rname = 'kerberos-org.mit.edu'
 
 # Verify forward resolution before testing for it.
 try:
@@ -82,6 +82,7 @@ if canonname.lower() != fname:
               '%s forward resolves to %s, not %s' % (oname, canonname, fname))
 
 # Test forward-only canonicalization (rdns=false).
+mark('rdns=false')
 testnr(oname, fname, 'R1')
 testnr(oname + ':123', fname + ':123', 'R1')
 testnr(oname + ':xyZ', fname + ':xyZ', 'R1')
@@ -91,11 +92,14 @@ try:
     names = socket.getnameinfo(sockaddr, socket.NI_NAMEREQD)
 except socket.gaierror:
     skip_rest('reverse sn2princ tests', 'cannot reverse resolve %s' % oname)
-if names[0].lower() != rname:
+rname = names[0].lower()
+if rname == fname:
     skip_rest('reverse sn2princ tests',
-              '%s reverse resolves to %s, not %s' % (oname, names[0], rname))
+              '%s reverse resolves to %s '
+              'which should be different from %s' % (oname, rname, fname))
 
 # Test default canonicalization (forward and reverse lookup).
+mark('default')
 test(oname, rname, 'R3')
 test(oname + ':123', rname + ':123', 'R3')
 test(oname + ':xyZ', rname + ':xyZ', 'R3')
