@@ -60,76 +60,18 @@ TYPEDEF_FUNC(
 
 #define pGetModuleFileNameEx pGetModuleFileNameExA
 
-// leash functions
-TYPEDEF_FUNC(
-    long,
-    WINAPIV,
-    not_an_API_LeashKRB4GetTickets,
-    (TICKETINFO *, TicketList **)
-    );
-TYPEDEF_FUNC(
-    long,
-    WINAPIV,
-    not_an_API_LeashAFSGetToken,
-    (TICKETINFO *, TicketList **, char *)
-    );
-TYPEDEF_FUNC(
-    long,
-    WINAPIV,
-    not_an_API_LeashGetTimeServerName,
-    (char *, const char*)
-    );
-
-extern DECL_FUNC_PTR(not_an_API_LeashKRB4GetTickets);
-extern DECL_FUNC_PTR(not_an_API_LeashAFSGetToken);
-extern DECL_FUNC_PTR(not_an_API_LeashGetTimeServerName);
 extern DECL_FUNC_PTR(Leash_kdestroy);
 extern DECL_FUNC_PTR(Leash_changepwd_dlg);
 extern DECL_FUNC_PTR(Leash_changepwd_dlg_ex);
 extern DECL_FUNC_PTR(Leash_kinit_dlg);
 extern DECL_FUNC_PTR(Leash_kinit_dlg_ex);
 extern DECL_FUNC_PTR(Leash_timesync);
-extern DECL_FUNC_PTR(Leash_get_default_lifetime);
-extern DECL_FUNC_PTR(Leash_set_default_lifetime);
-extern DECL_FUNC_PTR(Leash_get_default_forwardable);
-extern DECL_FUNC_PTR(Leash_set_default_forwardable);
-extern DECL_FUNC_PTR(Leash_get_default_renew_till);
-extern DECL_FUNC_PTR(Leash_set_default_renew_till);
-extern DECL_FUNC_PTR(Leash_get_default_noaddresses);
-extern DECL_FUNC_PTR(Leash_set_default_noaddresses);
-extern DECL_FUNC_PTR(Leash_get_default_proxiable);
-extern DECL_FUNC_PTR(Leash_set_default_proxiable);
-extern DECL_FUNC_PTR(Leash_get_default_publicip);
-extern DECL_FUNC_PTR(Leash_set_default_publicip);
-extern DECL_FUNC_PTR(Leash_get_default_use_krb4);
-extern DECL_FUNC_PTR(Leash_set_default_use_krb4);
-extern DECL_FUNC_PTR(Leash_get_default_life_min);
-extern DECL_FUNC_PTR(Leash_set_default_life_min);
-extern DECL_FUNC_PTR(Leash_get_default_life_max);
-extern DECL_FUNC_PTR(Leash_set_default_life_max);
-extern DECL_FUNC_PTR(Leash_get_default_renew_min);
-extern DECL_FUNC_PTR(Leash_set_default_renew_min);
-extern DECL_FUNC_PTR(Leash_get_default_renew_max);
-extern DECL_FUNC_PTR(Leash_set_default_renew_max);
-extern DECL_FUNC_PTR(Leash_get_default_renewable);
-extern DECL_FUNC_PTR(Leash_set_default_renewable);
-extern DECL_FUNC_PTR(Leash_get_lock_file_locations);
-extern DECL_FUNC_PTR(Leash_set_lock_file_locations);
 extern DECL_FUNC_PTR(Leash_get_default_uppercaserealm);
 extern DECL_FUNC_PTR(Leash_set_default_uppercaserealm);
 extern DECL_FUNC_PTR(Leash_get_default_mslsa_import);
-extern DECL_FUNC_PTR(Leash_set_default_mslsa_import);
-extern DECL_FUNC_PTR(Leash_get_default_preserve_kinit_settings);
-extern DECL_FUNC_PTR(Leash_set_default_preserve_kinit_settings);
 extern DECL_FUNC_PTR(Leash_import);
 extern DECL_FUNC_PTR(Leash_importable);
 extern DECL_FUNC_PTR(Leash_renew);
-extern DECL_FUNC_PTR(Leash_reset_defaults);
-
-////Do we still need this one?
-#define pLeashKRB4GetTickets     pnot_an_API_LeashKRB4GetTickets
-#define pLeashAFSGetToken        pnot_an_API_LeashAFSGetToken
-#define pLeashGetTimeServerName  pnot_an_API_LeashGetTimeServerName
 
 // psapi functions
 extern DECL_FUNC_PTR(GetModuleFileNameExA);
@@ -181,6 +123,7 @@ extern DECL_FUNC_PTR(krb5_get_renewed_creds);
 extern DECL_FUNC_PTR(krb5_cc_initialize);
 extern DECL_FUNC_PTR(krb5_cc_store_cred);
 extern DECL_FUNC_PTR(krb5_cc_get_full_name);
+extern DECL_FUNC_PTR(krb5_free_string);
 extern DECL_FUNC_PTR(krb5_enctype_to_name);
 extern DECL_FUNC_PTR(krb5_cc_get_type);
 extern DECL_FUNC_PTR(krb5int_cc_user_set_default_name);
@@ -231,13 +174,6 @@ extern DECL_FUNC_PTR(profile_release_string);
 
 #define LEASH_HELP_FILE "leash.chm"
 
-extern int  config_boolean_to_int(const char *);
-extern BOOL SetRegistryVariable(const CString& regVariable,
-                                const CString& regValue,
-                                const char* regSubKey = "Software\\MIT\\Leash32\\Settings");
-extern VOID LeashErrorBox(LPCSTR errorMsg, LPCSTR insertedString,
-                          LPCSTR errorFlag = "Error");
-
 // Get ticket info for the default ccache only
 extern void LeashKRB5ListDefaultTickets(TICKETINFO *ticketinfo);
 // clean up ticket info
@@ -248,26 +184,10 @@ extern void LeashKRB5ListAllTickets(TICKETINFO **ticketinfolist);
 // clean up ticket info list
 extern void LeashKRB5FreeTickets(TICKETINFO **ticketinfolist);
 
-
-
-class Directory
-{
-    CHAR m_savCurPath[MAX_PATH];
-    CString m_pathToValidate;
-
-public:
-    Directory(LPCSTR pathToValidate);
-    virtual ~Directory();
-
-    BOOL IsValidDirectory();
-    BOOL IsValidFile();
-};
-
 class TicketInfoWrapper {
   public:
     HANDLE     lockObj;
     TICKETINFO Krb5;
-    TICKETINFO Afs;
 };
 extern TicketInfoWrapper ticketinfo;
 
