@@ -1,6 +1,5 @@
+/* windows/lib/cacheapi.h */
 /*
- *  $Id: cacheapi.h 23100 2009-10-31 00:48:38Z tlyu $
- *
  * Copyright 1997 by the Regents of the University of Michigan
  *
  * This software is being provided to you, the LICENSEE, by the
@@ -91,7 +90,6 @@ typedef cc_int32  cc_time_t;
 #define CC_ERR_CACHE_FULL    15
 #define CC_ERR_CRED_VERSION  16
 
-
 /*
 ** types, structs, & constants
 */
@@ -104,21 +102,6 @@ typedef struct opaque_dll_control_block_type* apiCB;
 typedef struct opaque_ccache_pointer_type* ccache_p;
 typedef struct opaque_credential_iterator_type* ccache_cit;
 
-#if 0
-enum _cc_data_type {
-    type_ticket = 0,                /* 0 for ticket, second_ticket */
-    /* Ted's draft spec says these are to be
-       "as defined in the Kerberos V5 protocol"
-       all I can find are typdefs,
-       can't find an enumerated type or #define
-    */
-    type_address,           /* =  <"as defined in the Kerberos V5 protocol"> */
-    type_authdata,          /* = <"as defined in the Kerberos V5 protocol"> */
-    type_encryption,        /* = <"as defined in the Kerberos V5 protocol"> */
-    cc_data_type_max        /* for validation */
-};
-#endif
-
 typedef struct _cc_data
 {
     cc_uint32       type;		// should be one of _cc_data_type
@@ -143,7 +126,6 @@ typedef struct _cc_creds {
     cc_data **  authdata;
 } cc_creds;
 
-
 // begin V4 stuff
 // use an enumerated type so all callers infer the same meaning
 // these values are what krbv4win uses internally.
@@ -204,7 +186,6 @@ typedef struct _infoNC {
     cc_int32  vers;
 } infoNC;
 
-
 /*
 ** The official (externally visible) API
 */
@@ -237,7 +218,6 @@ cc_get_change_time(
     cc_time_t* time      // <  time of last change to main cache
     );
 
-
 /*
 ** Named Cache (NC) routines
 **   create, open, close, destroy, get_principal, get_cred_version, &
@@ -373,7 +353,6 @@ cc_lock_request(
                                 //   lock types
     );
 
-
 /*
 ** Credentials routines (work within an NC)
 ** store, remove_cred, seq_fetch_creds
